(* W07 *)
type t = int list * int list * int list;;
let debug = ref false;;
exception Peg_empty;;
exception Peg_too_small;;

let debugging _  = 
 debug := not (!debug);
 Format.printf "Debugging %sabled" (if !debug then "en" else "dis");
;;

(* print current state *)
let column ls = try
 let lines = (Lst.map (fun i -> Picture.row (Strng.of_int i)) ls) in
 let width = Lst.foldr (fun p -> max (Picture.width p)) 1 lines in
 let align p = Picture.beside (Picture.empty (width - Picture.width p) (Picture.height p)) p in
 Picture.stack (Lst.map align lines)
 with
 | _ -> (1,0,[[]])
;;

let rec print = function
 | ([],[],[]) -> Format.printf "- - -\n";
 | (l1,l2,l3) -> 
 let columns = Lst.map column [l1;l2;l3] in
 let height = Lst.foldr (fun p -> max (Picture.height p)) 1 columns in
 let align p = Picture.above (Picture.empty (Picture.width p) (height - Picture.height p)) p in
 let p = Picture.spread_with 1 (List.map align columns) in
 Format.printf "%a\n" Picture.toplevel_printer p;
;;

(*make problem*)
let make n = (IntLst.range 1 (n+1),[],[]);;

let rec solved = function
 | ([],[],[]) -> true
 | ([],[],[x]) -> true
 | ([],[],x::y::ys) when x <= y -> solved ([],[],y::ys)
 | _ -> false
;;

let map_i i f (l1,l2,l3) = match i with
 | 1 -> (f l1,l2,l3)
 | 2 -> (l1,f l2,l3)
 | 3 -> (l1,l2,f l3)
;;

let ith i (l1,l2,l3) = match i with
 | 1 -> l1
 | 2 -> l2
 | 3 -> l3
;;

let push_on_peg elt peg = match peg with
 | [] -> [elt]
 | x::_ when elt < x -> elt::peg 
 | _ -> raise Peg_too_small
;;

let top_from_peg = function
 | x::_ -> x
 | _ -> raise Peg_empty

let pop_from_peg = function
 | _::xs -> xs
 | _ -> raise Peg_empty

let push elt i ts = map_i i (push_on_peg elt) ts;;
let top j ts = top_from_peg (ith j ts);;
let pop j ts = map_i j pop_from_peg ts;;

let f ts = if !debug then print ts;;

let move i j ts = f ts; push (top i ts) j (pop i ts);;

