(*************************************************************
 This is a small formalization of lists in the theorem prover
 Isabelle 2013-1 (availabe from http://isabelle.in.tum.de/)
*************************************************************)

theory MyList 
imports Presburger
begin

datatype 'a list = Nil ("[]") | Cons 'a "'a list" (infixr "::" 65)

fun append :: "'a list \<Rightarrow> 'a list \<Rightarrow> 'a list" (infixr "@" 64) where
   "[]      @ ys = ys"
 | "(x::xs) @ ys = x::(xs@ys)" 

(* append is associative *)
lemma "xs @ (ys @ zs) = (xs @ ys) @ zs" proof (induct xs)
 case Nil
  show ?case unfolding append.simps(1) ..
 next
 case (Cons w ws) 
  show ?case unfolding append.simps(2) unfolding Cons ..
 qed

(* or shorter *)
lemma append_assoc: "xs @ (ys @ zs) = (xs @ ys) @ zs" by (induct xs) auto

(* some more simple lemmas *)
fun reverse :: "'a list \<Rightarrow> 'a list" where
   "reverse [] = []"
 | "reverse (x::xs) = reverse xs @ (x::[])"

lemma right_neutral: "xs @ []= xs" by (induct xs) auto

lemma "reverse (xs@ys) = reverse ys @ reverse xs" proof (induct xs)
 case Nil
  show ?case unfolding append.simps reverse.simps right_neutral ..
 next
 case (Cons w ws)
  show ?case unfolding append.simps reverse.simps Cons append_assoc ..
qed

lemma "reverse (xs@ys) = reverse ys @ reverse xs" proof (induct xs)
 case Nil thus ?case by (simp add: right_neutral)
 next
 case Cons thus ?case by (simp add: append_assoc)
qed

end
